document.addEventListener('DOMContentLoaded', function () {
    GetRunning(server);
}, false);

function AppViewModel() {
    var self = this;
    
    self.loading = ko.observable(true);
    
    self.sessions = ko.observableArray([]);
    
    self.session = ko.observable({
        guid: ko.observable(),
        rstatus: ko.observable(false),
        lstatus: ko.observable(false),
        istatus: ko.observable(false),
		estatus: ko.observable(false),
        messages: ko.observableArray(),
        executionStatus: ko.observable("Idle"),
        breakingStatus: ko.observable(""),
        eventStatus: ko.observable("Idle")
    });
    
    self.addSession = function (newGuid, address, port) {
        var session = {
            guid: ko.observable(newGuid),
            rstatus: ko.observable(false),
            lstatus: ko.observable(false),
            istatus: ko.observable(false),
			estatus: ko.observable(false),
            rsocket: ko.observable(),
            lsocket: ko.observable(),
            isocket: ko.observable(),
			esocket: ko.observable(),
            messages: ko.observableArray(),
            sessionMode: ko.observable(false),
            executionStatus: ko.observable("Idle"),
            breakingStatus: ko.observable(""),
            eventStatus: ko.observable("Idle"),
            url: ko.observable(address),
            port: ko.observable(port),
        };
        
        if (newGuid === "sessionMode") {
            var inputs = document.querySelectorAll('.proxyOnly');
            
            for (var i = 0; i < inputs.length; i++)
            inputs[i].hidden = true;
            
            session.sessionMode(true);
        }
        
        self.sessions.push(session);
        self.switchSess(session);
        connectLogHub(newGuid);
        connectEventHub(newGuid);
        connectResultHub(newGuid);
        connectInteractionHub(newGuid);
    };

    self.getUrl = function (){
        return self.session().url + ":" + self.session().port;
    }
    
    self.switchSession = function () {
        self.switchSess(this);
    };
    self.switchSess = function (session) {
        self.session(session);
    };
    
    self.removeSession = function () {
        var index = self.sessions.indexOf(this);
        
        disconnectInteractionHub(this.guid());
        disconnectEventHub(this.guid());
        disconnectResultHub(this.guid());
        disconnectLogHub(this.guid());
        
        deleteSession(this.guid());
        self.sessions.remove(this);
        
        if (this !== self.session())
        return;
        
        if (self.sessions().length === 0)
        return;
        // do nothing
        else if (self.sessions().length - 1 < index)
            self.switchSess(self.sessions()[self.sessions().length - 1]);
        else
            self.switchSess(self.sessions[index]);
    };
}

var viewModel = new AppViewModel();

ko.applyBindings(viewModel);


////////////// HTTP Requests


var client = new XMLHttpRequest();
var server = "127.0.0.1:20116";

function GetRunning(server){
  viewModel.loading(true);
    
    var url = "http://" + server + "/session";
    client.open("GET", url, true);
    
    client.onreadystatechange = function(){
 if (this.readyState === 4) {
        viewModel.loading(false);
        if (client.status === 0) {
            document.getElementById("status").innerText = "Server down?";
        }
        else {
            try {
                var json = JSON.parse(client.responseText);
                if(isArray(json)){
                    // Proxy with session IDs in array
                    console.log("Proxy detected");
                    for (i = 0; i < json.length; i++) {
                        viewModel.addSession(json[i]);
                    }
                    connectProxyWebsockets(server); 
                }else {
                    // Session
                    console.log("Session detected");
                        viewModel.addSession("sessionMode");
                }
            } catch{
                console.log("could not parse json");
            }
        }
    }
}

    client.onload = function () {
        var responseText = client.responseText;
        // process the response.
    };
    
    client.onerror = function () {
        var responseText = client.responseText;
    };
    
    client.send("");
}

function isJson(str) {
    try {
        JSON.parse(str);
    } catch (e) {
        return false;
    }
    return true;
}

function isArray(what) {
    return Object.prototype.toString.call(what) === '[object Array]';
}

function postSession(imageId) {
    viewModel.loading(true);
    
    var url = "http://" + server + "/session";
    
    if (imageId !== null || imageId === 'clone')
    url = url + "/" + imageId;
    
    client.open("POST", url, true);
    
    client.onreadystatechange = readySession;
    client.setRequestHeader('Content-Type', 'application/json');
    
    client.onload = function () {
        var responseText = client.responseText;
        console.log(responseText);
        // process the response.
    };
    
    client.onerror = function () {
        console.log('There was an error!');
        var responseText = client.responseText;
        console.log(responseText);
    };
    
    client.send("");
}

function deleteSession(guid) {
    var url = "http://" + server + "/session/" + guid;
    
    client.open("DELETE", url, true);
    
    client.onreadystatechange = reqReady;
    client.setRequestHeader('Content-Type', 'plain/text');
    
    client.send("");
}

function readySession() {
    
    if (this.readyState === 4) {
        viewModel.loading(false);
        if (client.status === 0) {
            document.getElementById("status").innerText = "Server down?";
        }
        else {
            var guidOrSession = "sessionMode";
            try {
                JSON.parse(client.responseText)
            } catch{
                guidOrSession = client.responseText;
            }
            console.log(guidOrSession);
            //viewModel.addSession(guidOrSession);
        }
    }
}

function reqReady() {
    
    
    if (this.readyState === 4) {
        viewModel.loading(false);
        document.getElementById("status").innerText = client.statusText + ":" + client.status;
        
        if (isJson(client.responseText)) {
            document.getElementById("responseText").value = JSON.stringify(JSON.parse(client.responseText), null, 2);
        } else {
            document.getElementById("responseText").value = client.responseText;
        }
        
    }
}

function load() {
    viewModel.loading(true);
    var x = document.getElementById("myFile");
    
    // Get the file
    var f = x.files[0];
    
    // Prepare Request
    var url = "http://" + server + "/plan";
    client.open("POST", url, true);
    client.setRequestHeader('id', viewModel.session().guid());
    client.setRequestHeader('Content-Type', 'text/plain');
    client.onreadystatechange = reqReady;
    
    
    // Read file and send contents
    if (f) {
        var r = new FileReader();
        r.onload = function (e) {
            var testPlan = e.target.result;
            client.send(testPlan);
        };
        r.readAsText(f);
    } else {
        alert("Failed to load file");
    }
    x.value = "";
}

function getExternal() {
    standardReq("http://" + server + "/plan/externalparameters", "GET", [{
    'key': "Content-Type",
    'value': "plain/text"
}], null);
}

function setExternal() {
    var url = "http://" + server + "/plan/externalparameters";
    
    var body = document.getElementById("responseText").value;
    
    standardReq(url, "POST", [{ 'key': "Content-Type", 'value': "application/json" }], body);
}

function postPlan() {
    var url = "http://" + server + "/plan/steps/";
    
    var body = document.getElementById("responseText").value;
    
    standardReq(url, "POST", [{ 'key': "Content-Type", 'value': "application/json" }], body);
}

function getPlan() {
    standardReq("http://" + server + "/plan/steps/", "GET", [], null);
}

function getStep() {
    var stepId = document.getElementById("stepId").value;
    var url = "http://" + server + "/plan/step/" + stepId;
    
    standardReq(url, "GET", [], null);
}

function getStepTypes() {
    var stepId = document.getElementById("typeStepId").value;
    var url = "http://" + server + "/steptypes/" + stepId;
    
    standardReq(url, "GET", [], null);
}

function setStep() {
    var stepId = document.getElementById("stepId").value;
    var url = "http://" + server + "/plan/step/" + stepId;
    
    var body = document.getElementById("responseText").value;
    
    standardReq(url, "POST", [{ 'key': "Content-Type", 'value': "application/json" }], body);
}

function getSettings() {
    var url = "http://" + server + "/settings";
    
    standardReq(url, "GET", [], null);
}

function getErrors() {
    var url = "http://" + server + "/plan/validation";
    
    standardReq(url, "GET", [], null);
}

function postSettings() {
    var url = "http://" + server + "/settings";
    
    var body = document.getElementById("responseText").value;
    
    standardReq(url, "POST", [{ 'key': "Content-Type", 'value': "application/json" }], body);
}

function getPlanXML() {
    standardReq("http://" + server + "/plan", "GET", [], null);
}

function runPlan() {
    setJumpVisibility(false);
    standardReq("http://" + server + "/plan/run", "POST", [], null);
}

function statusPlan() {
    standardReq("http://" + server + "/plan/run", "GET", [], null);
}

function abortPlan() {
    setJumpVisibility(false);
    standardReq("http://" + server + "/plan/run/abort", "POST", [], null);
}

function getToggle() {
    standardReq("http://" + server + "/plan/breakpoint", "GET", [], null);
}

function setToggle() {
    var stepId = document.getElementById("toggleStepId").value;
    var url = "http://" + server + "/plan/breakpoint/" + stepId;
    standardReq(url, "POST", [], null);
}

function setJumpVisibility(display) {
    if (display === false) {
        var x = document.getElementById("jumpToStepId");
        x.style.visibility = "hidden";
        x = document.getElementById("setJumpToStepId");
        x.style.visibility = "hidden";
    }
    else {
        var y = document.getElementById("jumpToStepId");
        y.style.visibility = "initial";
        y = document.getElementById("setJumpToStepId");
        y.style.visibility = "initial";
    }
}

function setJumpStep() {
    var stepId = document.getElementById("jumpToStepId").value;
    var url = "http://" + server + "/plan/jumptostep/" + stepId;
    standardReq(url, "POST", [], null);
}

function pauseNextStep() {
    standardReq("http://" + server + "/plan/breaknext", "POST", [], null);
    setJumpVisibility(false);
}

function getSessionInfo() {
    standardReq("http://" + server + "/session", "GET", [], null);
}

function postSessionInfo() {
    var body = document.getElementById("responseText").value;
    standardReq("http://" + server + "/session", "POST", [{ 'key': "Content-Type", 'value': "application/json" }], body);
}

function imageTemplate() {
    document.getElementById("responseText").value = `{
        "Packages":[
            {"Name": "Demonstration", "Version": "beta"}
        ],
        "Repositories": [ 
            "http://packages.opentap.keysight.com",
            "http://packages.opentap.io"
        ]
    }
    `
}

function getProxySettings() {
    var url = "http://" + server + "/settings";
    
    standardProxyReq(url, "GET", [], null);
}

function postProxySettings() {
    var url = "http://" + server + "/settings";
    
    var body = document.getElementById("responseText").value;
    
    standardProxyReq(url, "POST", [{ 'key': "Content-Type", 'value': "application/json" }], body);
}

function getSessions() {
    var url = "http://" + server + "/session";
    
    standardProxyReq(url, "GET", [], null);
}

function addSession(){
    var sessionId = document.getElementById("sessionId").value;
    viewModel.addSession(sessionId);
}

function getImage() {
    var imageId = document.getElementById("imageId").value;
    standardProxyReq("http://" + server + "/images/" + imageId, "GET", [], null);
}

function getImages() {
    standardProxyReq("http://" + server + "/images", "GET", [], null);
}

function getProxyImage() {
    standardProxyReq("http://" + server + "/image", "GET", [], null);
}

function postProxyImage() {
    var url = "http://" + server + "/image";
    
    var body = document.getElementById("responseText").value;
    
    standardProxyReq(url, "POST", [{ 'key': "Content-Type", 'value': "application/json" }], body);
}

function getSessionImage() {
    standardReq("http://" + server + "/image", "GET", [], null);
}

function postSessionImage() {
    var url = "http://" + server + "/image";
    
    var body = document.getElementById("responseText").value;
    
    standardReq(url, "POST", [{ 'key': "Content-Type", 'value': "application/json" }], body);
}


function postImage() {
    var body = document.getElementById("responseText").value;
    standardProxyReq("http://" + server + "/images", "POST", [{ 'key': "Content-Type", 'value': "application/json" }], body);
}

function createSessionFromImage() {
    var imageId = document.getElementById("imageId").value;
    postSession(imageId);
}

function modifyProxy() {
    var imageId = document.getElementById("proxyImageId").value;
    standardProxyReq("http://" + server + "/images/proxy/" + imageId, "POST", [], null);
}

function shutdownProxy() {
    var body = document.getElementById("responseText").value;
    standardProxyReq("http://" + server + "/proxy", "DELETE", [{ 'key': "Content-Type", 'value': "text/plain" }], body);
}

function getProxyId() {
    standardProxyReq("http://" + server + "/images/proxy", "GET", [], null);
}

function standardReq(url, method, headers, body) {
    viewModel.loading(true);
    client.open(method, url, true);
    client.onreadystatechange = reqReady;
    client.setRequestHeader('id', viewModel.session().guid());
    for (var i = 0; i <= headers.length - 1; i++) {
        client.setRequestHeader(headers[i].key, headers[i].value);
    }
    
    client.send(body);
}

function standardProxyReq(url, method, headers, body) {
    viewModel.loading(true);
    client.open(method, url, true);
    client.onreadystatechange = reqReady;
    for (var i = 0; i <= headers.length - 1; i++) {
        client.setRequestHeader(headers[i].key, headers[i].value);
    }
    
    client.send(body);
}

///////////////////// WebSockets

function connectProxyWebsockets(server) {
    var uri = "ws://" + server + "/proxyevents";
    var socket = new WebSocket(uri);
    
    socket.onopen = function (event) {
        console.log("Proxy Websockets opened");
    };
    socket.onclose = function (event) {
        console.log("Proxy Websockets closed");
    };
    socket.onmessage = function (event) {
        var json = JSON.parse(event.data);
        switch (json.Status) {
            case "SessionStarted":
                viewModel.addSession(json.Session.Id, json.Session.Address, json.Session.Port);
            break;
            case "SessionStopped":
                session = viewModel.sessions().find(s => s.guid() == json.Session.Id);
                if(session !== undefined)
                    viewModel.sessions.remove(session);
                break;
        }
    };
    socket.onerror = function (event) {
        console.log("Proxy Websockets error");
    };
    
}


function connectResultHub(guid) {
    var sessions = viewModel.sessions();
    var session = sessions.filter(function (item) {
        return item.guid() === guid;
    })[0];
    
    var id = session.guid();
    
    var uri = "ws://" + server + "/results";
    if (id !== "sessionMode") {
        uri = uri + "?id=" + id;
    }
    if(document.querySelector('#resultsData:checked') !== null){
        uri = uri + "?data=true";
    }
    console.log(uri);
    var socket = new WebSocket(uri);
    
    socket.onopen = function (event) {
        session.rstatus(true);
    };
    socket.onclose = function (event) {
        session.rstatus(false);
    };
    var lastBreakTestStepId = "";
    socket.onmessage = function (event) {
        var json = JSON.parse(event.data);
        var statusText = "";
        var breaking = false;
        var finishedTestStepId = "";
        switch (json.Status) {
            case "TestStepRunStart":
            statusText = 'Test step "' + json.TestStepName + '" started';
            break;
            case "TestStepRunCompleted":
                statusText = 'Test step "' + json.TestStepName + '" completed' + ' (' + json.Duration + ')';
                finishedTestStepId = json.TestStepId;
                break;
            case "TestPlanRunStart":
                statusText = 'Test plan "' + json.TestPlanName + '" started';
                break;
            case "TestPlanRunCompleted":
                statusText = 'Testplan run completed with verdict: ' + json.Verdict + ' (' + json.Duration + ')';
                break;
            case "Breaking...":
                statusText = 'Testplan is breaking on Step' + json.TestStepName + ' - ' + json.TestStepId + ' (' + json.Duration + ')';
                lastBreakTestStepId = json.TestStepId;
                breaking = true;
                break;
        }
        if (breaking) {
            session.breakingStatus(statusText);
            setJumpVisibility(true);
        }
        else {
            if (lastBreakTestStepId === finishedTestStepId)
                session.breakingStatus("");
            session.executionStatus(statusText);
        }
        console.log(json);
    };
    socket.onerror = function (event) {
        session.rstatus("Error: " + event.data);
    };
    
    session.rsocket(socket);
}

function disconnectResultHub(guid) {
    var sessions = viewModel.sessions();
    var session = sessions.filter(function (item) {
        return item.guid() === guid;
    })[0];
    
    
    session.rsocket().close();
}

function connectLogHub(guid) {
    var sessions = viewModel.sessions();
    var session = sessions.filter(function (item) {
        return item.guid() === guid;
    })[0];
    
    var id = session.guid();
    
    var uri = "ws://" + server + "/logs";
    if (id !== "") {
        uri = uri + "?id=" + id;
    }
    
    console.log(uri);
    var socket = new WebSocket(uri);
    
    socket.onopen = function (event) {
        session.lstatus(true);
    };
    socket.onclose = function (event) {
        session.lstatus(false);
    };
    socket.onmessage = function (event) {
        var json = JSON.parse(event.data);
        var result = json.Source + ' : ' + json.Message;
        session.messages.push(result);
        var objDiv = document.getElementById("socketResponseDiv");
        objDiv.scrollTop = objDiv.scrollHeight;
        console.log(json);
    };
    socket.onerror = function (event) {
        session.lstatus("Error: " + event.data);
    };
    
    session.lsocket(socket);
}

function disconnectLogHub(guid) {
    var sessions = viewModel.sessions();
    var session = sessions.filter(function (item) {
        return item.guid() === guid;
    })[0];
    
    session.lsocket().close();
}



function connectInteractionHub(guid) {
    var sessions = viewModel.sessions();
    var session = sessions.filter(function (item) {
        return item.guid() === guid;
    })[0];
    
    var id = session.guid();
    
    var uri = "ws://" + server + "/interactions";
    if (id !== "") {
        uri = uri + "?id=" + id;
    }
    
    console.log(uri);
    var socket = new WebSocket(uri);
    
    socket.onopen = function (event) {
        session.istatus(true);
    };
    socket.onclose = function (event) {
        session.istatus(false);
    };
    socket.onmessage = function (event) {
        var json = JSON.parse(event.data);

        var interactionDiv = document.getElementById("interactionDiv");
        
        if(json.InteractionEnded !== undefined){
            interactionDiv.classList.add("hidden");
            return;
        }
        
        interactionDiv.classList.remove("hidden");

        if (isJson(event.data)) {
            document.getElementById("interactionText").value = JSON.stringify(JSON.parse(event.data), null, 2);
        } else {
            document.getElementById("interactionText").value = event.data;
        }
        
        document.getElementById("interactionButton").onclick = function(){
            socket.send(document.getElementById("interactionText").value);
            interactionDiv.classList.add("hidden");
            viewModel.loading(false);
        }
    };
    socket.onerror = function (event) {
        session.istatus("Error: " + event.data);
    };
    
    session.isocket(socket);
}

function disconnectInteractionHub(guid) {
    var sessions = viewModel.sessions();
    var session = sessions.filter(function (item) {
        return item.guid() === guid;
    })[0];
    
    session.isocket().close();
}

function connectEventHub(guid) {
    var sessions = viewModel.sessions();
    var session = sessions.filter(function (item) {
        return item.guid() === guid;
    })[0];
    
    var id = session.guid();
    
    var uri = "ws://" + server + "/events";
    if (id !== "") {
        uri = uri + "?id=" + id;
    }
    console.log(uri);
    var socket = new WebSocket(uri);
    
    socket.onopen = function (event) {
        session.estatus(true);
    };
    socket.onclose = function (event) {
        session.estatus(false);
    };
    
    socket.onmessage = function (event) {
        var json = JSON.parse(event.data);
        var statusText = "";
        switch (json.EventType) {
            case "TestStepChanged":
            statusText = 'Test step "' + json.StepName + '" ' + json.StepId + ' changed';
            break;
            case "TestPlanChanged":
            statusText = 'Test Plan ' + json.PlanName + ' changed';
        }
        session.eventStatus(statusText);
        console.log(json);
    };
    socket.onerror = function (event) {
        session.estatus("Error: " + event.data);
    };
    
    session.esocket(socket);
}

function disconnectEventHub(guid) {
    var sessions = viewModel.sessions();
    var session = sessions.filter(function (item) {
        return item.guid() === guid;
    })[0];
    
    
    session.esocket().close();
}

function EventHub() {
    var session = viewModel.session();
    if (session.estatus() === false)
    connectEventHub(session.guid());
    else
    disconnectEventHub(session.guid());
}

function ResultHub() {
    var session = viewModel.session();
    if (session.rstatus() === false)
    connectResultHub(session.guid());
    else
    disconnectResultHub(session.guid());
}

function LogHub() {
    var session = viewModel.session();
    if (session.lstatus() === false)
    connectLogHub(session.guid());
    else
    disconnectLogHub(session.guid());
}

function InteractionHub() {
    var session = viewModel.session();
    if (session.istatus() === false)
    connectInteractionHub(session.guid());
    else
    disconnectInteractionHub(session.guid());
}